// Allan Spale
// CS 488
// Assignment 4: Raytracing


#include "colors.inc"  
#include "stones.inc"
#include "stars.inc"
#include "textures.inc"
#include "glass.inc"   
#include "golds.inc"            
#include "metals.inc" 

//--------------------- ORRERY PART AND PLANET DEFINITIONS -----------------------\\
//
// Different objects are defined in order to create objects
// for use in other objects but also to make the code easier to read.
//
// Note that the "ambient" property for "finish" must explicitly be defined as 0 in
// order to eliminate its effects on "lighting" the world.
//
// All references to locations of objects in relation to other objects
// are given by the location of the camera


// Golden round thing that goes around a "stick"
#declare Base = 
	cylinder { <0,0,0> <0,1,0>, 1 
		texture { T_Gold_1A } 
	}  


// Horizontal silver tube that connects things		
#declare HorizStick = 
	cylinder { <0,0,0> <1,0,0> 0.5
		texture { T_Silver_1A } 	
	}                      

// Vertical silver tube that connects things	
#declare VertStick = 
	cylinder { <0,0,0> <0,1,0> 0.5 
		texture { T_Silver_1A } 	
	}

	
// Sphere for planets and moons...called "planet" for simplicity of reference	           
#declare Planet = 
	sphere { <0, 0, 0>, 1 }
	

#declare Sun =  union {
	object {
		// Place where Saturn connects 
		Base
		scale <6,8,6>
		translate <0,7,0>		
	}
	object {
		// Place where Jupiter connects 
		Base
		scale <5,7,5>
		translate <0,16,0>		
	}
	object { 
		// Place where Mars connects
		Base
		scale <4,6,4>
		translate <0,24,0>
	}
	object { 
		// Place where Earth connects
		Base
		scale <3,5,3>
		translate <0,31,0>
	}
	object { 
		// Place where Venus connects
		Base
		scale <2,4,2>
		translate <0,37,0> 
		
	}
	object { 
		// Place where Mercury connects
		Base
		scale <1,3,1>
		translate <0,42,0>
	}     
	object { 
		// Stick where all the bases go 
		VertStick  
		scale <1,46,1> 
		translate <0,0,-0.5> 
	}
	object { 
		// Sun 
		Planet  
		scale <9,9,9>  
		translate <0,54.5,0>  
		texture { T_Gold_1A } 
	}                         
}
	

#declare Mercury = union {
	object {
		// Stick representing the distance Mercury is from the Sun
		HorizStick
		scale <15.7,1,1>
		translate <0.5,46.5,0>    
	}
	object { 
		VertStick  
		scale <1,10,1> 
		translate <16.45,46.5,0> 
	}
	object { 
		// Mercury
		Planet  
		scale <3.02, 3.02, 3.02>  
		translate <16.45,59.02,0>
	 	texture { T_Orange_Glass }
	}                  
}
	
	
#declare Venus = union {
	object {
		// Stick representing the distance Venus is from the Sun
		HorizStick
		scale <19,1,1>
		translate <1,42,0>    
	}
	object { 
		VertStick  
		scale <1,10,1> 
		translate <20.25,42,0> 
	}
	object { 
		// Venus 
		Planet  
		scale <3.39,3.39,3.39>  
		translate <20.25,55.39,0>
	 	pigment { P_Brass1 }
	 	finish { F_MetalB }
	}
}        

	    
#declare Moon = union {
	object {
		// Stick representing the distance Moon is from the Earth
		HorizStick
		scale <6,1,1>
		translate <0.5,7,0>    
	}
	object { 
		VertStick
		scale <1,5,1> 
		translate <6.5,7,0> 
	}
	object { 
		// Moon 
		Planet
		scale <2,2,2>  
		translate <6.5,14,0>  
		texture { T_Silver_5E }
		pigment { P_Silver3 }
		finish { F_MetalE }		 
	}
}        


#declare Earth = union {
	object {
		// Stick representing the distance Earth is from the Sun
		HorizStick
		scale <21,1,1>
		translate <1.5,35.5,0>    
	}
	object { 
		VertStick  
		scale <1,10,1> 
		translate <22.75,35.5,0> 
	}
	object { 
		// Earth
		Planet  
		scale <3.42,3.42,3.42>  
		translate <22.75,48.42,0>  
		texture {T_Winebottle_Glass}
	}
	object { 
		// Place where Moon connects 
		Base
		scale <1,2,1>
		translate <22.75,41.5,0>  
	}

	object {
		Moon 
		//rotate <0,-135,0> 
		translate <22.75,35.5,0>
	}	
}


#declare Mars = union {
	object {
		// Stick representing the distance Mars is from the Sun
		HorizStick
		scale <27.4,1,1>
		translate <4,28,0>    
	}
	object { 
		VertStick  
		scale <1,10,1> 
		translate <31.65,28,0> 
	}
	object { 
		// Mars 
		Planet  
		scale <2.65,2.65,2.65>  
		translate <31.65,40.65,0>  
		texture { T_Ruby_Glass } 
	}
}       


#declare Io = union {
	object {
		// Stick representing the distance Io is from the Jupiter
		HorizStick
		scale <9,1,1>
		translate <0.5,8.5,0>    
	}
	object { 
		VertStick  
		scale <1,5,1> 
		translate <9.5,8.5,0> 
	}
	object { 
		// Io 
		Planet  
		scale <2,2,2>  
		translate <9.5,15.5,0>  
		texture { T_Silver_5E }
		pigment { P_Silver3 }
		finish { F_MetalE }

	}
}        


#declare Europa = union {
	object {
		// Stick representing the distance Europa is from the Jupiter
		HorizStick
		scale <10,1,1>
		translate <0.5,8.5,0>    
	}
	object { 
		VertStick  
		scale <1,5,1> 
		translate <10.5,8.5,0> 
	}
	object { 
		// Europa
		Planet  
		scale <2,2,2>  
		translate <10.5,15.5,0>  
		texture { T_Silver_5E }
		pigment { P_Silver3 }
		finish { F_MetalE }
	}
}          

#declare Ganymede = union {
	object {
		// Stick representing the distance Ganymede is from the Jupiter
		HorizStick
		scale <11,1,1>
		translate <0.5,8.5,0>    
	}
	object { 
		VertStick  
		scale <1,5,1> 
		translate <11.5,8.5,0> 
	}
	object { 
		// Ganymede 
		Planet  
		scale <2,2,2>  
		translate <11.5,15.5,0>  
		texture { T_Silver_5E }
		pigment { P_Silver3 }
		finish { F_MetalE }
	}
}          
              
  
#declare Callisto = union {
	object {
		// Stick representing the distance Callisto is from the Jupiter
		HorizStick
		scale <12,1,1>
		translate <0.5,8.5,0>    
	}
	object { 
		VertStick  
		scale <1,5,1> 
		translate <12.5,8.5,0> 
	}
	object { 
		// Callisto
		Planet  
		scale <2,2,2>  
		translate <12.5,15.5,0>  
		texture { T_Silver_5E }
		pigment { P_Silver3 }
		finish { F_MetalE }
	}
}
 


#declare Jupiter = union {
	object {
		// Stick representing the distance Jupiter is from the Sun
		HorizStick
		scale <34.3,1,1>
		translate <2.5,20.5,0>    
	}
	object { 
		VertStick  
		scale <1,10,1> 
		translate <37.05,20.5,0> 
	}
	object { 
		// Jupiter 
		Planet  
		scale <6.75,6.75,6.75>  
		translate <37.05,37.25,0>  		
		pigment { P_Copper3 }  
		finish { F_MetalC }
	}
	object { 
		// Place where Io connects  
		Base
		scale <1,2,1>
		translate <37.05,29.0,0>  
	}
	object {
		Io
		rotate <0,90,0>  
		//rotate <0,180,0> 
		translate <37.05,21.0,0>
	}	
	
	object { 
		// Place where Europa connects 
		Base
		scale <1,2,1>
		translate <37.05,26.5,0>  
	}		
	object {
		Europa 
		//rotate <0,0,0>
		rotate <0,30,0>  
		translate <37.05,18.5,0>
	}	
	object { 
		// Place where Ganymede connects 
		Base
		scale <1,2,1>
		translate <37.05,24.0,0>  
	}
	object {
		Ganymede 
		//rotate <0,-270,0> 
		rotate <0,-30,0> 
		translate <37.05,16.0,0>
	}	  
	object { 
		Base
		scale <1,2,1>
		translate <37.05,21.5,0>  
	}		
	object {
		// Place where Callisto connects 
		Callisto 
		//rotate <0,-90,0> 
		rotate <0,-90,0> 
		translate <37.05,13.0,0>
	}	
}        


#declare Titan = union {
	object {
		// Stick representing the distance Titan is from the Saturn
		HorizStick
		scale <8,1,1>
		translate <0.5,7,0>    
	}
	object { 
		VertStick  
		scale <1,5,1> 
		translate <8.5,7,0> 
	}
	object { 
		// Titan 
		Planet  
		scale <2,2,2>  
		translate <8.5,14,0>  
		texture { T_Silver_5E }
		pigment { P_Silver3 }
		finish { F_MetalE }
	}
} 


#declare Saturn = union {
	object {
		// Stick representing the distance Saturn is from the Sun
		HorizStick
		scale <41.2,1,1>
		translate <3.0,12.0,0>    
	}
	object { 
		VertStick  
		scale <1,10,1> 
		translate <44.45,12.0,0> 
	}
	object { 
		// Saturn 
		Planet  
		scale <6.13,6.13,6.13>  
		translate <44.45,28.38,0>
		texture { T_Gold_4A  }
	}
	// Ring of Saturn
	torus { 
		9.0, 1        
		rotate <0,0,-30>
		translate <44.45,28.38,0> 
		texture { T_Yellow_Glass }
	}
	object { 
		Base
		scale <1,2,1>
		translate <44.45,18,0>  
	}
	object {
		Titan 
		//rotate <0,135,0> 
		translate <44.45,12,0>
	}	
}


// Object used for the orrery platform
#declare Surface = union {
	box {
	  <-60, -0.1, -60>  
	  < 60, -3.1,  60>  
	  finish { 
	  	specular .9  
	  	reflection .3 
	  }
	  color ForestGreen    
	  //texture { T_Grnt15 }
	}		
}        


// crank
#declare Crank = union {
	object {
		// attached to base of orrery
		HorizStick    
		scale <64,2,2>
		//scale <43.7,2,2>
		translate <0.0, 2.5, 0.0>    
	}  
	object {                                         	
		// "joint" for holding the handle to the stick
		Base
		rotate <90,90,0>
		scale <3,3,3>
		//translate <42.7, 2.5, 0.0>		
		translate <63, 2.5, 0.0>
	}	
	object { 
		// stick that attaches to handle
		VertStick  
		scale <2,15,2> 
		//translate <43.95,2.5,0.0> 
		translate <64.25,2.5,0.0>
	}
	cone { 
		// handle
	    //<43.45,18.5,0.0>, 		/* base point */
	    <63.75,18.5,0.0>
	    2,      				/* base radius */
	    //<63,18.5, 0.0>, 		/* cap point */
	    <83,18.5,0.0>	    
	    4        				/* cap radius */
	    texture { T_Gold_1A  }   
	}
}


//------------------------------ WORLD SETUP -------------------------\\

// camera location with orthographic view
// centers orrery and part of platform in the viewport

   
//// HANDLE DOES NOT GO THROUGH DOME ANGLE
/*camera {   
	orthographic
	location <0, 0, -82>
	look_at  <63,2.5,  0>
}
*/

//ORIGINAL
camera {   
	perspective
	location <0, 57, 102>
	look_at  <0, 30,  0>
} 


/*
// OVERHEAD
camera {   
	perspective
	location <0, 138, 0>
	look_at  <0, 0, 0>
}
*/
 

/*
// HANDLE
camera {   
	perspective
	rotate <0,60,0>
	location <-70,27.5, 0.0>
	look_at  <0, 35, 0>
} 
*/
  
/*
// OFF-ANGLE OVERHEAD
camera {   
	orthographic
	rotate <0,-225,0>	
	location <16.75,68.42,0>
	look_at  <0,0,0>
}
*/


// "Bright flourescent" area light
light_source {
	<0, 100, 0>
	color White
	area_light <-100, 0, 0>, <0, 0, 100>, 5,5
	adaptive 1
	jitter
}      

light_source {
	<0, -100, 0>
	color White
	area_light <-100, 0, 0>, <0, 0, 100>, 5,5
	adaptive 1
	jitter
}      

light_source {
	<0, 0, 100>
	color White
	area_light <-100, 0, 0>, <0, 100, 0>, 5,5
	adaptive 1
	jitter
}      

light_source {
	<0, 0, -100>
	color White
	area_light <-100, 0, 0>, <0, 100, 0>, 5,5
	adaptive 1
	jitter
}     

                 
//----------------------------- OBJECTS IN THE WORLD --------------------------\\
                      
//============================ Orrery Base =========================\\
cone
{
    <0,0,0>, 					/* base point */
    15,      					/* base radius */
    <0,3,0>, 					/* cap point */
    7        					/* cap radius */   
    scale <2,2,1>
	texture { PinkAlabaster  }
	finish {
      reflection .45
      specular .8
      roughness 0.1
    }	
}


//============================ Orrery Dome =========================\\
difference {
	sphere {
		<0,22,0>, 59.5
	   //<0,22,0>, 20 
	   //scale <3.5,2,2.3>
	   
	   texture {T_Glass3}
	   texture {Starfield6}
	   pigment { color <.8,.8,1> filter 1 }
	   finish {
	      ambient 0
	      diffuse 0
	      reflection .2
	      specular .8
	      roughness .001
	   } 
	}

	// part of dome that meets the base
	box { 
		<-63, 6, -63>, 
		<63, -66, 63>
	   pigment { color <.8,.8,1> filter 1 }
	   finish {
	      ambient 0
	      diffuse 0
	      reflection .2
	      specular .8
	      roughness .001
	   } 
	}


}
     
      
//============================ Orrery Platform ======================\\
union {
	object { Surface rotate <0,45,0> } 
	object { Surface rotate <0,55,0>  translate <0,-3,0> }
	object { Surface rotate <0,65,0>  translate <0,-6,0> } 
	object { Surface rotate <0,75,0>  translate <0,-9,0> }
	object { Surface rotate <0,85,0>  translate <0,-12,0> } 
	object { Surface rotate <0,95,0>  translate <0,-15,0> }
	object { Surface rotate <0,105,0> translate <0,-18,0> } 
	object { Surface rotate <0,115,0> translate <0,-21,0> }
 	object { Surface rotate <0,125,0> translate <0,-24,0> } 
}    
     
     
//============================ Orrery Crank ======================\\
object { Crank  rotate <0,30,0> }


//============================ Planets and Their Moons ======================\\
object { Sun  translate <0,3,0>}
object { Mercury  rotate <0,-15,0> }
object { Venus  rotate <0,-35,0>  }
object { Earth  rotate <0,-55,0>  }
object { Mars  rotate <0,-85,0>  }
object { Jupiter  rotate <0,-125,0> } 
object { Saturn  rotate <0,-165,0> }


  sky_sphere {
    pigment {
      bozo
      turbulence 0.65
      octaves 3
      omega 0.7
      lambda -0.78
      color_map {
          [0.0 0.1 color rgb <0.85, 0.85, 0.85>
                   color rgb <0.75, 0.75, 0.75>]
          [0.1 0.5 color rgb <0.75, 0.75, 0.75>
                   color rgbt <1, 1, 1, 1>]
          [0.5 1.0 color rgbt <1, 1, 1, 1>
                   color rgbt <1, 1, 1, 1>]
      }
      scale <0.2, 0.5, 0.2>
    }
    rotate -135*x
  }
